"""
Echoview Code Operator source file
========================================================================================

Created by Echoview® 13.1.113

See the Echoview help file for Code operator documentation and examples.

NumPy User Guide is available from here: https://docs.scipy.org/doc/numpy/user/
NumPy Reference is available from here: https://docs.scipy.org/doc/numpy/reference/
SciPy Reference Guide is available from here: https://docs.scipy.org/doc/scipy/reference/
"""

# Authorship information
__author__ = "Echoview Software Pty Ltd"
__copyright__ = "Copyright 2023"

# System Imports
from typing import List

# Libraries
import echoview as ev
import numpy as np
from scipy.signal import convolve2d


class Operator(ev.OperatorBase):

    """Generalize Echoview's XxY convolution operator's Gaussian blur.
    You can now specify the centroid and standard deviation for the
    Gaussian function within the kernel.

    Operands
    ----------------------------------------------------------------------------------------

    * Operand 1 - Single beam data in dB.

    Notes
    ----------------------------------------------------------------------------------------

    For best results, ensure that
    self.kernel_column_span < Window size (pings)
    self.kernel_row_span < The number of ping samples in range.
    """

    # Refer to the __init__ example in appendix B.2.2 "Programming an
    # attribute for the Code operator" to set up this method.
    def __init__(self):

        # Number of pings for the kernel to span (odd integer), k1.
        '''Kernel column span of 3.'''
        # Number of samples in the range dimension for the kernel to span (odd integer), k2.
        '''Kernel row span of 3.'''

        # Gaussian offset in the ping dimension, 𝜇x.
        '''Gaussian x offset of 0.'''
        # Gaussian offset in the range dimension, 𝜇y.
        '''Gaussian y offset of 0.'''
        # Gaussian standard deviation in the ping dimension, σx.
        '''Gaussian x standard deviation of 1.'''
        # Gaussian standard deviation in the range dimension, σy.
        '''Gaussian y standard deviation of 1.'''

    def eval(self, inputs: List[ev.OperandInput]):

        # Refer to section 2.1.2 "About the commands in the default
        # Echoview Python source file" to assign Operand 1 to the
        # variable first_input.
        first_input = '''Operand 1'''

        # Refer to the example in section 13.2 "Ping sample data from
        # all pings in the window of measurement", to use list
        # comprehension to collate the ping sample data. Then try to
        # work out the shape of the ping_samples_db array. If you get
        # stuck, refer to section 13.3 "The shape of an array".
        ping_samples_db = ['''Sample data for each ping in the window of measurements.''']

        # The formula to convert from dB to linear is 10^(X/10), where
        # X is the ping sample data in dB.
        # Hint: Use the np.power and np.divide functions. (Attempt the
        # computation in a single line of code.)
        ping_samples_linear = '''Convert the ping sample data from dB to linear.'''

        # Generate the kernel based on a 2D Gaussian.
        # Hint: This function takes in the attributes you have
        # programmed in __init__.
        kernel = normalized_gaussian_kernel('''kernel_column_span''', '''kernel_row_span''', '''offset_x''',
                                            '''offset_y''', '''standard_deviation_x''', '''standard_deviation_y''')

        # Use the function from SciPy to perform the convolution.
        convolution_linear = convolve2d('''Ping data''', '''kernel''', mode='same')

        # The formula to convert from linear to dB is 10*log10(X), where
        # X is the ping sample data in linear.
        # Hint: Use the np.log10 and np.multiply functions. (Attempt the
        # computation in a single line of code.)
        convolution_db = '''Convert the sample data from linear to dB.'''

        # Return the matched ping.
        # Refer to section 13.2 "Ping sample data from all pings in
        # the window of measurement" for an example.
        return '''The matched ping.'''


# ====================================================================
#    User-defined functions for eval.
#

def normalized_gaussian_kernel('''k1''', '''k2''', '''𝜇𝑥''', '''𝜇𝑦''', '''σx''', '''σy'''):
    # Generate the x and y coordinates based on the span of the
    # kernel.
    x_coords, y_coords = kernel_coordinates('''Number of columns.''', '''Number of rows.''')

    # Return a kernel based on a normalized Gaussian function.
    # Refer to equation 2 in section 17.1 "Algorithm".
    return '''Normalized Gaussian function.'''


def kernel_coordinates('''k1''', '''k2'''):
    # Set the limits based on the number of rows and columns.
    # Hint: Since kernel_column_span and kernel_row_span are always
    # odd numbers, you can compute the x and y limits using (N-1)/2.
    x_limit = '''Deduce the positive x limit from kernel_column_span.'''
    y_limit = '''Deduce the positive y limit from kernel_row_span.'''

    # Generate coordinates symmetric about the origin.
    # Hint: Traverse from the negative to the positive limit by the
    # number of columns (x) or rows (y).
    kernel_x_coords = np.linspace('''Start''', '''Stop''', num='''Number of steps.''')
    kernel_y_coords = np.linspace('''Start''', '''Stop''', num='''Number of steps.''')

    # Return the x and y coordinate permutations.
    return np.meshgrid('''x coordinates''', '''y coordinates''', indexing='ij')
