"""
Echoview Code Operator source file
========================================================================================

Created by Echoview® 13.1.113

See the Echoview help file for Code operator documentation and examples.

NumPy User Guide is available from here: https://docs.scipy.org/doc/numpy/user/
NumPy Reference is available from here: https://docs.scipy.org/doc/numpy/reference/
SciPy Reference Guide is available from here: https://docs.scipy.org/doc/scipy/reference/
"""

# Authorship information
__author__ = "Echoview Software Pty Ltd"
__copyright__ = "Copyright 2023"

# System Imports
from typing import List

# Libraries
import echoview as ev
import numpy as np


class Operator(ev.OperatorBase):

    """Create a circular mask for multibeam pings.

    Operands
    ----------------------------------------------------------------------------------------

    * Operand 1 - Multibeam data.
    """

    # Refer to the __init__ example in appendix B2.2 "Programming an
    # attribute" for the syntax for programming this method.
    def __init__(self):
        # Circular mask's coordinates (r_0, phi) and radius R.
        '''Range of 6 meters to the center of the circular mask.'''  # r_0 (meters)
        '''Angle of -6 degrees to the center of the circular mask.'''  # phi (degrees)
        '''Radius of 0.5 meters for the circular mask.'''  # R (meters)

    def eval(self, inputs):

        # Refer to section 2.1.2 "About the commands in the default
        # Echoview Python source file" to assign Operand 1 to the
        # variable first_input.
        first_input = '''Operand 1'''

        # Refer to section 9.2 "Utilizing the window of measurements"
        # to get the matched ping.
        ping = '''The matched ping.'''

        # Hint: Multibeam data has shape (range, beam). Use the
        # np.shape method to obtain the number of samples from the
        # range axis of the matched ping.
        # Refer to section 13.3 "The shape of an array".
        sample_count = '''Number of samples.'''

        # For a given beam, compute the range coordinate of each
        # sample as the midpoint of that sample's thickness.
        # Hint: Refer to the attributes of the Measurement class at
        # https://support.echoview.com/WebHelp/How_To/Use_The_Code_Operator/Using_the_Code_operator.htm#class_Measurement
        # to obtain the start and stop range of the ping.
        sample_thickness = ('''Subtract the stop range from the start range.''')/sample_count
        half_thickness = sample_thickness/2
        # Hint: Use half_thickness to traverse from a half-step
        # forward from the start range to a half-step backward from
        # the stop range.
        range_values_1D = np.linspace('''Start''', '''Stop''', num='''Number of steps.''')

        # At a given range, obtain the angle coordinate of each
        # beam in radians.
        # Hint: Refer to the attributes of the Measurement class at
        # https://support.echoview.com/WebHelp/How_To/Use_The_Code_Operator/Using_the_Code_operator.htm#class_Measurement
        # to obtain the beam angles. Convert these from degrees to
        # radians.
        beam_angles_1D = '''Angle coordinate at the midpoint of each beam in radians.'''

        # Create a lattice representing the coordinates (r_i, theta_i)
        # of each sample in the ping's fan.
        sample_ranges, sample_angles = np.meshgrid('''r_i values''', '''theta_i values''', indexing='ij')

        # Compute |r_0 - r_i| by finding the difference in the x and y
        # components.
        # Hints:
        # - First convert phi (from __init__) to radians.
        # - Then use the np.sin and np.cos methods to change all the
        #   coordinates from polar (r, theta) to cartesian (x, y).
        # - Finally use np.sqrt to compute the distance.
        center_angle_radians = '''Convert phi to radians.'''
        distance_x = '''r_0*cos(phi) - r_i*cos(theta_i)'''
        distance_y = '''r_0*sin(phi) - r_i*sin(theta_i)'''
        distance_2D = '''Square root of (distance_x**2 + distance_y**2)'''

        # Set ping samples to no data when |r_0 - r_i| < R.
        # Hints:
        # - Use the np.where method. Refer to appendix D "Examples
        #   using numpy.where" for some examples on using np.where.
        # - np.nan is no data in Echoview.
        return '''Ping sample data with samples within the mask set to no data.'''
